package com.xdf.creative.shiro;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.xdf.creative.base.vo.creative.SysRoleQueryVo;
import com.xdf.creative.enums.ApiCode;
import com.xdf.creative.module.entity.SysUser;
import com.xdf.creative.module.service.SysPermissionService;
import com.xdf.creative.module.service.SysRoleService;
import com.xdf.creative.module.service.SysUserService;
import com.xdf.creative.shiro.jwt.JwtToken;
import com.xdf.creative.shiro.jwt.JwtUtil;
import com.xdf.creative.support.exception.BusinessException;
import com.xdf.creative.util.StringUtil;
import com.xdf.creative.util.page.ApiResult;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class CustomRealm extends AuthorizingRealm {

    private final static Logger log = LoggerFactory.getLogger(AuthorizingRealm.class);

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysPermissionService sysPermissionService;

    /**
     * 必须重写此方法，不然Shiro会报错
     *
     * @param token
     * @return
     */
    @Override
    public boolean supports(AuthenticationToken token) {
        return token instanceof JwtToken;
    }


    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        //获取登录用户名
        String token = (String) principalCollection.getPrimaryPrincipal();
        //根据用户名去数据库查询用户信息
        SysUser user = null;
        try {
            user = sysUserService.getSysUserByUserName(JwtUtil.getUsername(token));
            if (null == user) {
                return null;
            }
        } catch (Exception e) {
            throw new BusinessException("用户不存在");
        }
        //添加角色和权限
        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
        HashSet<Long> roleIdSet = new HashSet<>();
        HashSet<String> roleNameSet = new HashSet<>();
        HashSet<String> roleCodeSet = new HashSet<>();

        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(JwtUtil.getUserId(token));
        if (CollectionUtil.isEmpty(sysRoleQueryVoList)) {
            throw new BusinessException("角色不存在");
        }
        sysRoleQueryVoList.stream().forEach(sysRoleQueryVo -> {
            roleIdSet.add(sysRoleQueryVo.getId());
            roleNameSet.add(sysRoleQueryVo.getName());
            roleCodeSet.add(sysRoleQueryVo.getCode());
        });
        simpleAuthorizationInfo.setRoles(roleCodeSet);
        Set<String> permissionSrt = null;
        try {
            permissionSrt = sysPermissionService.getPermissionUrlByUserId(JwtUtil.getUserId(token));
        } catch (Exception e) {
            throw new BusinessException("资源不存在");
        }
        simpleAuthorizationInfo.setStringPermissions(permissionSrt);
        return simpleAuthorizationInfo;
    }

    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        //加这一步的目的是在Post请求的时候会先进认证，然后在到请求
        if (authenticationToken.getPrincipal() == null) {
            return null;
        }
        String token = (String) authenticationToken.getCredentials();
        //token失效
        if (JwtUtil.isExpired(token)) {
            log.error("令牌过期");
            String result = JSON.toJSONString(ApiResult.fail(ApiCode.TOKEN_INVALID));
            throw new AuthenticationException(result);
        }
        String userName = JwtUtil.getUsername(token);
        if (StringUtil.isEmpty(userName)) {
            log.error("非法令牌");
            String result = JSON.toJSONString(ApiResult.fail(ApiCode.TOKEN_INVALID));
            throw new AuthenticationException(result);
        }
        //根据用户名去数据库查询用户信息
        SysUser user = null;
        //获取用户信息
        try {
            user = sysUserService.getSysUserByUserName(userName);
        } catch (Exception e) {
            log.error("用户 { " + userName + " } 不存在 ");
            throw new AccountException("账户不存在");
        }
        if (user == null) {
            //这里返回后会报出对应异常
            throw new BusinessException("用户不存在");
        } else {
            //这里验证authenticationToken和simpleAuthenticationInfo的信息
            SimpleAuthenticationInfo simpleAuthenticationInfo = new SimpleAuthenticationInfo(token, token, getName());
            return simpleAuthenticationInfo;
        }
    }


}

